function panasonic_t(driver) {
    var queue = [];
    var currentState = -1;
    var toTimerID = -1;

    var STATE_SET_POWER = 1;
    var STATE_SET_INPUT = 2;
    var STATE_GET_POWER = 3;
    var STATE_GET_INPUT = 4;

    IR.SetInterval(2000, poll);

    function poll() {
        var found = false;
        for (var i = 0; i < queue.length; i++) {
            if (queue[i].state == STATE_GET_POWER) found = true;
        }
        if (!found) getPowerState();

        found = false;
        for (var i = 0; i < queue.length; i++) {
            if (queue[i].state == STATE_GET_INPUT) found = true;
        }
        if (!found) getInputState();

    }

    function setPowerOn() {
        addCommandToQueue('PON', undefined, STATE_SET_POWER);
    }

    function setPowerOff() {
        addCommandToQueue('POF', undefined, STATE_SET_POWER);
    }

    function setInputHDMI() {
        addCommandToQueue('IIS', 'HD1', STATE_SET_INPUT);
    }

    function setInputSDI() {
        addCommandToQueue('IIS', 'SD1', STATE_SET_INPUT);
    }

    function addCommandToQueue(command, value, state) {
        queue.push({
            state: state,
            cmd: command,
            val: value
        });
        sendCommand();

    }

    function getPowerState() {
        addCommandToQueue('QPW', undefined, STATE_GET_POWER);
    }

    function getInputState() {
        addCommandToQueue('QIN', undefined, STATE_GET_INPUT);
    }

    function sendCommand() {
        if (currentState != -1 || queue.length == 0) return;
        var msg = queue.shift();
        currentState = msg.state;
        toTimerID = IR.SetTimeout(3000, timeout);
        var str = msg.cmd;
        if (msg.val != undefined) str += ":" + msg.val;
        IR.Log('PROJECTOR: Sent 00' + str);
        driver.Send(['00' + str + '\r']);
    }

    IR.AddListener(IR.EVENT_RECEIVE_TEXT, driver, function(text) {
        var randomNr = "";
        IR.Log("PROJECTOR: Received " + text);

        switch (currentState) {
            case STATE_SET_POWER:
            case STATE_GET_POWER:
                IR.Log("PROJECTOR: GET POWER " + text);
                var val = parseInt(text, 10);
                IR.SetVariable("Drivers.Projector.Power", val);
                break;
            case STATE_SET_INPUT:
            case STATE_GET_INPUT:
                IR.Log("PROJECTOR: GET INPUT " + text);
                if (text.indexOf('HD1') != -1) {
                    IR.SetVariable("Drivers.Projector.Input HDMI", 1);
                    IR.SetVariable("Drivers.Projector.Input SDI", 0);
                }
                if (text.indexOf('SD1') != -1) {
                    IR.SetVariable("Drivers.Projector.Input SDI", 1);
                    IR.SetVariable("Drivers.Projector.Input HDMI", 0);
                }

                break;
            case -1:

                break;
        }
        IR.ClearInterval(toTimerID);
        currentState = -1;
        sendCommand();
    });

    function timeout() {
        IR.Log("TIMEOUT");
        currentState = -1;
        sendCommand();
    }

    IR.AddListener(IR.EVENT_CHANNEL_SET, driver, function(name, value) {
        if (name == "Power on") setPowerOn();
        if (name == "Power off") setPowerOff();
        if (name == "Input SDI") setInputSDI();
        if (name == "Input HDMI") setInputHDMI();

    });
}

var p = new panasonic_t(IR.GetDevice("Projector"));