function qsc_t(driverName, config) {
    var device = IR.GetDevice(driverName);
    var queue = [];
    var buf = "";
    var currentEvent = -1;
    var online;
    var callbacks = [];

    var EVENT_CONTROL_SET_VALUE = 1;    
    var EVENT_CONTROL_GET = 2;
    var EVENT_STATUS_GET = 3;
    var EVENT_CONTROL_SET_VALUE_RELEASE = 4;
    //id above 100 = no response
    var EVENT_CONTROL_TRIGGER = 100;
    var EVENT_LOAD_SNAPSHOT = 101;


    for(var i = 0; i < config.controls.length; i++){
      config.controls[i].pending = 0;
    }

    IR.AddListener(IR.EVENT_RECEIVE_TEXT, device, function(text) {
        buf += text;
        //        IR.Log("Received: " + text);
        var index = buf.indexOf("\r\n");
        if (index != -1) {
            var response = buf.substring(0, index);
            buf = buf.substring(index + 2); //gooi de complete response weg
            IR.ClearInterval(timeoutTimerID);
            fireEvent(currentEvent, response);
            currentEvent = -1;
            sendData();
        }
    });
    IR.AddListener(IR.EVENT_ONLINE, device, function() {
        online = true;
    });

    IR.AddListener(IR.EVENT_OFFLINE, device, function() {
        online = false;
        queue = [];
        for (var i = 0; i < config.controls.length; i++) {
            config.controls[i].pending = 0;
        }
    });

    IR.SetInterval(800, periodicPoll); // vraag iedere x tijd de status van een groep op

    function periodicPoll() {
        for (var i = 0; i < config.controls.length; i++) {
            if (config.controls[i].pending == 0 || config.controls[i].pending == undefined) getControl(config.controls[i].qscLevel);
        }
    }

    function fireEvent(event, data) {
        if (callbacks[event] == undefined) return;
        for (var i = 0; i < callbacks[event].length; i++) {
            callbacks[event][i](data);
        }
    }

    function addListener(event, cb) {
        if (callbacks[event] == undefined) callbacks[event] = [];
        callbacks[event].push(cb);
    }

    var msg;

    function sendData() {
        if (queue.length == 0) return;
        msg = queue.shift();
        device.Send([msg.data + '\r\n']);
        //        IR.Log('Sending: ' + msg.data);
        if (msg.event < 100) {
            currentEvent = msg.event;
            if (timeoutTimerID != -1) IR.ClearInterval(timeoutTimerID);
            timeoutTimerID = IR.SetTimeout(5000, timeout);
        } else {
            sendData();
        }
    }

    var timeoutCtr = 0;
    var timeoutTimerID = -1;

    function timeout() {
        timeoutCtr++;
        if (timeoutCtr <= 3) {
            device.Send([msg.data + '\r\n']);
            timeoutTimerID = IR.SetTimeout(5000, timeout);
        } else {
            timeoutCtr = 0;
            currentEvent = -1;
            sendData(); //send next command
        }

    }

    function addToQueue(_data, _event) {

        if (!online) {
            //clear flag to get new value once back online
            //clear it here because of people can move the fader when system is offline
            for (var i = 0; i < config.controls.length; i++) {
                config.controls[i].pending = 0;
            }
            return; //alleen data verzenden als er een TCP connectie is

        }
        if (_event == EVENT_CONTROL_SET_VALUE) { //put first in line
            queue.splice(0, 0, {
                data: _data,
                event: _event
            });
        } else { 
            queue.push({
                data: _data,
                event: _event
            });
        }
        if (currentEvent == -1) {
            sendData();
        }
    }

    function controlTrigger(controlID) {
        addToQueue('ct ' + controlID, EVENT_CONTROL_TRIGGER);
    }

    function loadSnapshot(snapshotBank, number, ramptime) {
        addToQueue('ssl ' + snapshotBank + ' ' + number + ' ' + ramptime, EVENT_LOAD_SNAPSHOT);
    }

    function getStatus() {
        addToQueue('sg', EVENT_STATUS_GET);
    }

    function setControlValue(controlID, value) {
        addToQueue('csv "' + controlID + '" ' + parseFloat(value), EVENT_CONTROL_SET_VALUE);
    }

    function getControl(controlID) {
        addToQueue('cg "' + controlID + '"', EVENT_CONTROL_GET);
    }

    addListener(EVENT_STATUS_GET, function(data) {
        IR.Log('EVENT_STATUS !@# ' + data);
    });

    addListener(EVENT_CONTROL_GET, parseFaderLevel);
    addListener(EVENT_CONTROL_SET_VALUE, parseFaderLevel);

    function parseFaderLevel(data) {
          //  IR.Log('EVENT_CONTROL_GET !@# ' + data);
        var results = data.toString().split(' ');
        var controlName = results[1] + " " + results[2];
        controlName = controlName.replace(/\"/g, '');

        for (var i = 0; i < config.controls.length; i++) {
            if (config.controls[i].qscLevel == controlName) {
            
                if (config.controls[i].pending > 0 && currentEvent == EVENT_CONTROL_GET) return;
                if(config.controls[i].pending > 0) config.controls[i].pending--;
                else{
                getItem(config.controls[i].guiLevel).Value = results[4];
                getItem(config.controls[i].guiMute).Value = results[4];
                config.controls[i].last = results[4];
                }
                
            }
        }
    }

    this.controlTrigger = controlTrigger;
    this.loadSnapshot = loadSnapshot;
    this.setControlValue = setControlValue;

    ///////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////
    ///////////////////////////// GUI /////////////////////////////
    ///////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////// 

    function getItem(name) {
        var obj;
        var results = name.split('.');
        if (results.length == 2) {
            obj = IR.GetItem(results[0]).GetItem(results[1]);
        } else if (results.length == 1) {
            obj = IR.GetItem(results[0]);
        }

        return obj;
    }

    for (var i = 0; i < config.controls.length; i++) {
        addListenerToLevel(getItem(config.controls[i].guiLevel), config.controls[i].qscLevel, i);
        addListenerToMute(getItem(config.controls[i].guiMute), config.controls[i].qscLevel, i);
    }

    function addListenerToLevel(item, qscLevel, index) {
        var moved = false;
        var last = -100;
        IR.AddListener(IR.EVENT_ITEM_RELEASE, item, function() {
            if (!moved) return;
            moved = false;
            setLevel(qscLevel, item.Value, index);
        });

        
        
        
        IR.AddListener(IR.EVENT_TOUCH_MOVE, item, moveLevel);
        IR.AddListener(IR.EVENT_MOUSE_MOVE, item, moveLevel);

        function moveLevel() {
            moved = true;
            if (Math.abs(item.Value - last) > 5) {
                last = item.Value;     
                 setLevel(qscLevel, item.Value, index);        
         
            }
            
        }

        IR.AddListener(IR.EVENT_ITEM_PRESS, item, function() {
            item.Value = config.controls[index].last;
        });
    }
    
    function setLevel(qscLevel, value, index){           
           for(var i = 0; i < queue.length; i++){
               if(queue[i].data.indexOf(qscLevel) != -1){
                queue.splice(i,1);
                config.controls[index].pending--;
                i = 0;
               }
           }
            setControlValue(qscLevel, value);
            getItem(config.controls[index].guiMute).Value = value;
            config.controls[index].pending++; //flag to prevent level in gui to jump up and down when a message is pending
            config.controls[index].last = value;
        }

    function addListenerToMute(item, qscLevel, index) {
        IR.AddListener(IR.EVENT_ITEM_PRESS, item, function() {
        setLevel(qscLevel, item.Value, index);
            //setControlValue(qscLevel, item.Value);
            getItem(config.controls[index].guiLevel).Value = item.Value;
            //config.controls[index].pending++; //flag to prevent level in gui to jump up and down when a message is pending
        });
    }

}

var config = {
    controls: [
        
        //QSC pagina
        {
            guiLevel: "Q-SYS:main.level1",
            guiMute: "Q-SYS:main.btnMute1",
            qscLevel: "Mixer8x5Input1Gain (katheder-l)"
        },
              
        {
            guiLevel: "Q-SYS:main.level2",
            guiMute: "Q-SYS:main.btnMute2",
            qscLevel: "Mixer8x5Input2Gain (katherder-r)"
        },
        
        {
            guiLevel: "Q-SYS:main.level3",
            guiMute: "Q-SYS:main.btnMute3",
            qscLevel: "Mixer8x5Input3Gain (HH)"
        },
        {
            guiLevel: "Q-SYS:main.level4",
            guiMute: "Q-SYS:main.btnMute4",
            qscLevel: "Mixer8x5Input4Gain (lap)"
        },
        {
            guiLevel: "Q-SYS:main.level5",
            guiMute: "Q-SYS:main.btnMute5",
            qscLevel: "Mixer8x5Input5Gain (tascam)"
        },
        
        {
            guiLevel: "Q-SYS:main.level6",
            guiMute: "Q-SYS:main.btnMute6",
            qscLevel: "Mixer8x5Input6Gain (sonos)"
        },
        
        {
            guiLevel: "Q-SYS:main.level7",
            guiMute: "Q-SYS:main.btnMute7",
            qscLevel: "Mixer8x5Input7Gain (instument)"
        },
        {
            guiLevel: "Q-SYS:main.level8",
            guiMute: "Q-SYS:main.btnMute8",
            qscLevel: "Mixer8x5Input8Gain (skype)"
        },
        
        //Sonos pagina
        {
            guiLevel: "Sonos:main.level1",
            guiMute: "Sonos:main.btnMute1",
            qscLevel: "Mixer8x5Input6Gain (sonos)"
        },
        {
            guiLevel: "Sonos:main.level2",
            guiMute: "Sonos:main.btnMute2",
            qscLevel: "Mixer8x5Input1Gain (katheder-l)"
        },
        {
            guiLevel: "Sonos:main.level3",
            guiMute: "Sonos:main.btnMute3",
            qscLevel: "Mixer8x5Input2Gain (katherder-r)"
        },
        {
            guiLevel: "Sonos:main.level4",
            guiMute: "Sonos:main.btnMute4",
            qscLevel: "Mixer8x5Input5Gain (tascam)"
        },
             
    ]
}

var qsc = new qsc_t('qsc', config);

function autofaderuit() {
qsc.controlTrigger("autoFadeOutDisable");
/*
    qsc.setControlValue('autoFadeOutDisable', 1);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('autoFadeOutEnable', 0);
    });    
    */
}

function autofaderaan() {
qsc.controlTrigger("autoFadeOutEnable");
/*
    qsc.setControlValue('autoFadeOutDisable', 0);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('autoFadeOutEnable', 1);
    });    
    */
}

function sonosOpen() {
    qsc.setControlValue('SonosOpen', 1);
    IR.SetTimeout(1000, function(){    
      qsc.setControlValue('SonosOpen', 0);
    });
    
}

function katheder1Open() {
    qsc.setControlValue('Katherder1Open', 1);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('Katherder1Open', 0);
    });    
}

function katheder2Open() {
    qsc.setControlValue('Katherder2Open', 1);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('Katherder2Open', 0);
    });    
}

function LaptopOpen() {
    qsc.setControlValue('LaptopOpen', 1);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('LaptopOpen', 0);
    });    
}

function HandzenderOpen() {
    qsc.setControlValue('HandzenderOpen', 1);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('HandzenderOpen', 0);
    });    
}

function InstrumentOpen() {
    qsc.setControlValue('InstrumentOpen', 1);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('InstrumentOpen', 0);
    });    
}

function SkypeOpen() {
    qsc.setControlValue('SkypeOpen', 1);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('SkypeOpen', 0);
    });    
    
}

function CDOpen() {
    qsc.setControlValue('TascamOpen', 1);
    IR.SetTimeout(1000, function(){      
    qsc.setControlValue('TascamOpen', 0);
        });
}