// конструктор для iBezel, на входе ссылка на устройство
function iBezel(in_device) {
	var device = in_device;
	var refreshTimer;
   var aliveTimer;

	// конструктор
	function Init() {
		device.Connect();
	}

	// обработчик online
	function onOnline() {
		IR.Log("iBezel online");
      device.Send(["req:alive:1;"]);
      device.Send(["req:infupdate:standard;"]);
      device.Send(["req:volume;"]);
		// отправляем пакет alive каждые 150 секунд
		aliveTimer = IR.SetInterval(150000, function () {
				device.Send(["req:alive:1;"]);
			});
      refreshTimer = IR.SetInterval(5000, function () {
				device.Send(["req:infupdate:standard;"]);
			});
	}
	// обработчик offline
	function onOffline() {
		IR.Log("iBezel offline");
		IR.ClearInterval(aliveTimer);
      IR.ClearInterval(refreshTimer);
	}

   function messageRecogniser(command) {
		var param = command.split(":");
      
		switch (param[0]) {
		case "evn":
			// сработало событие
			if (param[1].indexOf("irevent") != -1) {
				device.SetFeedback("Motion", parseInt(param[2], 10));
			} else if (param[1].indexOf("button") != -1) {
				if (param[2] == "pushed") {
					device.SetFeedback(param[1], 1);
				} else {
					device.SetFeedback(param[1], 0);
				}
			} else if (param[1].indexOf("digital") != -1) {
				if (param[2] == "closed") {
					device.SetFeedback(param[1], 1);
				} else {
					device.SetFeedback(param[1], 0);
				}
			}
			break;
		case "inf":
			// сработал реквест
			switch (param[1]) {
			case "plugdrive":
				switch(param[2]) {
            case "docking":
               device.SetFeedback("StatusPlug", 1);
               break;
            case "undocking":
               device.SetFeedback("StatusPlug", 2);
               break;
            case "docked":
               device.SetFeedback("StatusPlug", 3);
               break;
            case "undocked":
               device.SetFeedback("StatusPlug", 4);
               break;
            }
				break;
			case "clackdrive":
				switch(param[2]) {
            case "closing":
               device.SetFeedback("StatusClack", 1);
               break;
            case "opening":
               device.SetFeedback("StatusClack", 2);
               break;
            case "closed":
               device.SetFeedback("StatusClack", 3);
               break;
            case "opened":
               device.SetFeedback("StatusClack", 4);
               break;
            }
				break;
			case "dockingstate":
				switch(param[2]) {
            case "docking":
            case "undocking":
               device.SetFeedback("StatusDock", 1);
               break;
            case "docked":
               device.SetFeedback("StatusDock", 0);
               break;
            case "undocked":
               device.SetFeedback("StatusDock", 2);
               break;
            }
				break;
			case "relay0":
				if (param[2] == "opened") {
					device.SetFeedback("StatusRelay", 0);
				} else {
					device.SetFeedback("StatusRelay", 1);
				}
				break;
			case "beep":
				device.SetFeedback("StatusBeep", parseInt(param[2], 10));
				break;
			case "irlevel":
				device.SetFeedback("StatusMotion", parseInt(param[2], 10));
				break;
			case "alive":
				/* MAYBE TODO */
				break;
			case "iBezelMode":
				/* MAYBE TODO */
				break;
         case "volume":
            var vol = parseInt(param[2], 10);
            if(vol == 0) {
               device.SetFeedback("Mute", 1);   
            } else {
               device.SetFeedback("Mute", 0);
               device.SetFeedback("Volume", vol);
            }
				break;
			default:
				if (param[1].indexOf("led") != -1) {
					device.SetFeedback(param[1], parseInt(param[2], 10));
				}
				break;
			}
			break;

		}
   }
	// обработчик информации от устройства
	function onReceive(text) {
		var commands = text.split(";");
      IR.Log("iBezel receive command: " + text);
      for(var i = 0; i < commands.length; i++) {
         messageRecogniser(commands[i]);
      }
   }
	// обработчик отправки информации устройству
	function onChannelSet(name, value) {
		IR.Log("iBezel channel (" + name + "), set to " + value);
		switch (name) {
		case "Beep":
			device.Send(["cmd:beep:" + value + ";"]);
			break;
		case "Dock":
			if (value == 1) {
				device.Send(["cmd:idock:open;"]);
			} else if (value == 0) {
				device.Send(["cmd:idock:close;"]);
			} else if (value == 2) {
				device.Send(["cmd:idock:toggle;"]);
			}
			break;
		case "Relay":
			if (value == 0) {
				device.Send(["cmd:relay0:open;"]);
			} else if (value == 1) {
				device.Send(["cmd:relay0:close;"]);
			} else if (value == 2) {
				device.Send(["cmd:relay0:toggle;"]);
			}
			break;
      case "Volume":
         device.SetFeedback("Mute", 0);
         device.SetFeedback("Volume", value);
			device.Send(["cmd:volume:" + value + ";"]);
			break;
      case "Mute":
         if(value) {
            device.Send(["cmd:volume:0;"]);
         } else {
            var old_vol = device.GetFeedback("Volume");
            device.Send(["cmd:volume:" + old_vol + ";"]);
         }
			break;
		default:
			if (name.indexOf("LED") != -1) {
				device.Send(["cmd:" + name.toLowerCase() + ":" + value + ";"]);
			}
			break;
		}
	}

	IR.AddListener(IR.EVENT_ONLINE, device, onOnline, this);
	IR.AddListener(IR.EVENT_OFFLINE, device, onOffline, this);
	IR.AddListener(IR.EVENT_RECEIVE_TEXT, device, onReceive, this);
	IR.AddListener(IR.EVENT_CHANNEL_SET, device, onChannelSet, this);

	// деструктор
	function Kill() {
		IR.RemoveListener(IR.EVENT_ONLINE, device, onOnline, this);
		IR.RemoveListener(IR.EVENT_OFFLINE, device, onOffline, this);
		IR.RemoveListener(IR.EVENT_RECEIVE_TEXT, device, onReceive, this);
		IR.RemoveListener(IR.EVENT_CHANNEL_SET, device, onChannelSet, this);
	}

	Init();
}
